@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -20,36 +20,34 @@
  ***************************************/
 
 
-#include <assert.h>
 #include <stdlib.h>
-#include <stdio.h>
 #include <string.h>
-#include <sys/stat.h>
 
 #include "types.h"
 #include "nodes.h"
-#include "segments.h"
 
+#include "typesx.h"
 #include "nodesx.h"
 #include "segmentsx.h"
 #include "waysx.h"
 
-#include "types.h"
-
 #include "files.h"
 #include "logging.h"
 #include "sorting.h"
 
 
-/* Variables */
+/* Global variables */
 
 /*+ The command line '--tmpdir' option or its default value. +*/
 extern char *option_tmpdirname;
 
-/*+ A temporary file-local variable for use by the sort functions. +*/
-static NodesX *sortnodesx;
+/* Local variables */
 
-/* Functions */
+/*+ Temporary file-local variables for use by the sort functions. +*/
+static NodesX *sortnodesx;
+static latlong_t lat_min,lat_max,lon_min,lon_max;
+
+/* Local functions */
 
 static int sort_by_id(NodeX *a,NodeX *b);
 static int deduplicate_and_index_by_id(NodeX *nodex,index_t index);
@@ -63,36 +61,43 @@ static int index_by_lat_long(NodeX *nodex,index_t index);
 
   NodesX *NewNodeList Returns a pointer to the node list.
 
-  int append Set to 1 if the file is to be opened for appending (now or later).
+  int append Set to 1 if the file is to be opened for appending.
+
+  int readonly Set to 1 if the file is to be opened for reading.
   ++++++++++++++++++++++++++++++++++++++*/
 
-NodesX *NewNodeList(int append)
+NodesX *NewNodeList(int append,int readonly)
 {
  NodesX *nodesx;
 
  nodesx=(NodesX*)calloc(1,sizeof(NodesX));
 
- assert(nodesx); /* Check calloc() worked */
+ logassert(nodesx,"Failed to allocate memory (try using slim mode?)"); /* Check calloc() worked */
 
- nodesx->filename=(char*)malloc(strlen(option_tmpdirname)+32);
+ nodesx->filename    =(char*)malloc(strlen(option_tmpdirname)+32);
+ nodesx->filename_tmp=(char*)malloc(strlen(option_tmpdirname)+32);
+
+ sprintf(nodesx->filename    ,"%s/nodesx.parsed.mem",option_tmpdirname);
+ sprintf(nodesx->filename_tmp,"%s/nodesx.%p.tmp"    ,option_tmpdirname,(void*)nodesx);
+
+ if(append || readonly)
+    if(ExistsFile(nodesx->filename))
+      {
+       off_t size;
+
+       size=SizeFile(nodesx->filename);
+
+       nodesx->number=size/sizeof(NodeX);
+
+       RenameFile(nodesx->filename,nodesx->filename_tmp);
+      }
 
  if(append)
-    sprintf(nodesx->filename,"%s/nodesx.input.tmp",option_tmpdirname);
+    nodesx->fd=OpenFileAppend(nodesx->filename_tmp);
+ else if(!readonly)
+    nodesx->fd=OpenFileNew(nodesx->filename_tmp);
  else
-    sprintf(nodesx->filename,"%s/nodesx.%p.tmp",option_tmpdirname,(void*)nodesx);
-
- if(append)
-   {
-    off_t size;
-
-    nodesx->fd=OpenFileAppend(nodesx->filename);
-
-    size=SizeFile(nodesx->filename);
-
-    nodesx->number=size/sizeof(NodeX);
-   }
- else
-    nodesx->fd=OpenFileNew(nodesx->filename);
+    nodesx->fd=-1;
 
  return(nodesx);
 }
@@ -103,15 +108,18 @@ NodesX *NewNodeList(int append)
 
   NodesX *nodesx The set of nodes to be freed.
 
-  int keep Set to 1 if the file is to be kept (for appending later).
+  int keep If set then the results file is to be kept.
   ++++++++++++++++++++++++++++++++++++++*/
 
 void FreeNodeList(NodesX *nodesx,int keep)
 {
- if(!keep)
-    DeleteFile(nodesx->filename);
+ if(keep)
+    RenameFile(nodesx->filename_tmp,nodesx->filename);
+ else
+    DeleteFile(nodesx->filename_tmp);
 
  free(nodesx->filename);
+ free(nodesx->filename_tmp);
 
  if(nodesx->idata)
     free(nodesx->idata);
@@ -119,6 +127,9 @@ void FreeNodeList(NodesX *nodesx,int keep)
  if(nodesx->gdata)
     free(nodesx->gdata);
 
+ if(nodesx->pdata)
+    free(nodesx->pdata);
+
  if(nodesx->super)
     free(nodesx->super);
 
@@ -139,10 +150,10 @@ void FreeNodeList(NodesX *nodesx,int keep)
 
   transports_t allow The allowed traffic types through the node.
 
-  uint16_t flags The flags to set for this node.
+  nodeflags_t flags The flags to set for this node.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void AppendNode(NodesX *nodesx,node_t id,double latitude,double longitude,transports_t allow,uint16_t flags)
+void AppendNodeList(NodesX *nodesx,node_t id,double latitude,double longitude,transports_t allow,nodeflags_t flags)
 {
  NodeX nodex;
 
@@ -156,7 +167,79 @@ void AppendNode(NodesX *nodesx,node_t id,double latitude,double longitude,transp
 
  nodesx->number++;
 
- assert(nodesx->number<NODE_FAKE); /* NODE_FAKE marks the high-water mark for real nodes. */
+ logassert(nodesx->number<NODE_FAKE,"Too many nodes (change index_t to 64-bits?)"); /* NODE_FAKE marks the high-water mark for real nodes. */
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Finish appending nodes and change the filename over.
+
+  NodesX *nodesx The nodes that have been appended.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void FinishNodeList(NodesX *nodesx)
+{
+ if(nodesx->fd!=-1)
+    nodesx->fd=CloseFile(nodesx->fd);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Find a particular node index.
+
+  index_t IndexNodeX Returns the index of the extended node with the specified id.
+
+  NodesX *nodesx The set of nodes to use.
+
+  node_t id The node id to look for.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+index_t IndexNodeX(NodesX *nodesx,node_t id)
+{
+ index_t start=0;
+ index_t end=nodesx->number-1;
+ index_t mid;
+
+ /* Binary search - search key exact match only is required.
+  *
+  *  # <- start  |  Check mid and move start or end if it doesn't match
+  *  #           |
+  *  #           |  Since an exact match is wanted we can set end=mid-1
+  *  # <- mid    |  or start=mid+1 because we know that mid doesn't match.
+  *  #           |
+  *  #           |  Eventually either end=start or end=start+1 and one of
+  *  # <- end    |  start or end is the wanted one.
+  */
+
+ if(end<start)                        /* There are no nodes */
+    return(NO_NODE);
+ else if(id<nodesx->idata[start])     /* Check key is not before start */
+    return(NO_NODE);
+ else if(id>nodesx->idata[end])       /* Check key is not after end */
+    return(NO_NODE);
+ else
+   {
+    do
+      {
+       mid=(start+end)/2;             /* Choose mid point */
+
+       if(nodesx->idata[mid]<id)      /* Mid point is too low */
+          start=mid+1;
+       else if(nodesx->idata[mid]>id) /* Mid point is too high */
+          end=mid?(mid-1):mid;
+       else                           /* Mid point is correct */
+          return(mid);
+      }
+    while((end-start)>1);
+
+    if(nodesx->idata[start]==id)      /* Start is correct */
+       return(start);
+
+    if(nodesx->idata[end]==id)        /* End is correct */
+       return(end);
+   }
+
+ return(NO_NODE);
 }
 
 
@@ -175,32 +258,29 @@ void SortNodeList(NodesX *nodesx)
 
  printf_first("Sorting Nodes");
 
- /* Close the file (finished appending) */
-
- nodesx->fd=CloseFile(nodesx->fd);
-
  /* Re-open the file read-only and a new file writeable */
 
- nodesx->fd=ReOpenFile(nodesx->filename);
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
 
- DeleteFile(nodesx->filename);
+ DeleteFile(nodesx->filename_tmp);
 
- fd=OpenFileNew(nodesx->filename);
+ fd=OpenFileNew(nodesx->filename_tmp);
 
  /* Allocate the array of indexes */
 
  nodesx->idata=(node_t*)malloc(nodesx->number*sizeof(node_t));
 
- assert(nodesx->idata); /* Check malloc() worked */
+ logassert(nodesx->idata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
- /* Sort by node indexes */
+ /* Sort the nodes by ID and index them */
 
  xnumber=nodesx->number;
- nodesx->number=0;
 
  sortnodesx=nodesx;
 
- filesort_fixed(nodesx->fd,fd,sizeof(NodeX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))deduplicate_and_index_by_id);
+ nodesx->number=filesort_fixed(nodesx->fd,fd,sizeof(NodeX),NULL,
+                                                           (int (*)(const void*,const void*))sort_by_id,
+                                                           (int (*)(void*,index_t))deduplicate_and_index_by_id);
 
  /* Close the files */
 
@@ -233,7 +313,7 @@ static int sort_by_id(NodeX *a,NodeX *b)
  else if(a_id>b_id)
     return(1);
  else
-    return(0);
+    return(-FILESORT_PRESERVE_ORDER(a,b)); /* latest version first */
 }
 
 
@@ -244,21 +324,170 @@ static int sort_by_id(NodeX *a,NodeX *b)
 
   NodeX *nodex The extended node.
 
-  index_t index The index of this node in the total.
+  index_t index The number of sorted nodes that have already been written to the output file.
   ++++++++++++++++++++++++++++++++++++++*/
 
 static int deduplicate_and_index_by_id(NodeX *nodex,index_t index)
 {
- if(index==0 || sortnodesx->idata[index-1]!=nodex->id)
+ static node_t previd=NO_NODE_ID;
+
+ if(nodex->id!=previd)
    {
-    sortnodesx->idata[index]=nodex->id;
+    previd=nodex->id;
 
-    sortnodesx->number++;
+    if(nodex->flags&NODE_DELETED)
+       return(0);
+    else
+      {
+       sortnodesx->idata[index]=nodex->id;
 
-    return(1);
+       return(1);
+      }
+   }
+ else
+    return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Remove any nodes that are not part of a highway.
+
+  NodesX *nodesx The set of nodes to modify.
+
+  SegmentsX *segmentsx The set of segments to use.
+
+  int keep If set to 1 then keep the old data file otherwise delete it.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void RemoveNonHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,int keep)
+{
+ NodeX nodex;
+ index_t total=0,highway=0,nothighway=0;
+ int fd;
+
+ /* Print the start message */
+
+ printf_first("Checking Nodes: Nodes=0");
+
+ /* Re-open the file read-only and a new file writeable */
+
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
+
+ if(keep)
+    RenameFile(nodesx->filename_tmp,nodesx->filename);
+ else
+    DeleteFile(nodesx->filename_tmp);
+
+ fd=OpenFileNew(nodesx->filename_tmp);
+
+ /* Modify the on-disk image */
+
+ while(!ReadFile(nodesx->fd,&nodex,sizeof(NodeX)))
+   {
+    if(!IsBitSet(segmentsx->usednode,total))
+       nothighway++;
+    else
+      {
+       nodex.id=highway;
+       nodesx->idata[highway]=nodesx->idata[total];
+
+       WriteFile(fd,&nodex,sizeof(NodeX));
+
+       highway++;
+      }
+
+    total++;
+
+    if(!(total%10000))
+       printf_middle("Checking Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
    }
 
- return(0);
+ nodesx->number=highway;
+
+ /* Close the files */
+
+ nodesx->fd=CloseFile(nodesx->fd);
+ CloseFile(fd);
+
+ /* Free the now-unneeded index */
+
+ free(segmentsx->usednode);
+ segmentsx->usednode=NULL;
+
+ /* Print the final message */
+
+ printf_last("Checked Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Remove any nodes that have been pruned.
+
+  NodesX *nodesx The set of nodes to prune.
+
+  SegmentsX *segmentsx The set of segments to use.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void RemovePrunedNodes(NodesX *nodesx,SegmentsX *segmentsx)
+{
+ NodeX nodex;
+ index_t total=0,pruned=0,notpruned=0;
+ int fd;
+
+ /* Print the start message */
+
+ printf_first("Deleting Pruned Nodes: Nodes=0 Pruned=0");
+
+ /* Allocate the array of indexes */
+
+ nodesx->pdata=(index_t*)malloc(nodesx->number*sizeof(index_t));
+
+ logassert(nodesx->pdata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
+
+ /* Re-open the file read-only and a new file writeable */
+
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
+
+ DeleteFile(nodesx->filename_tmp);
+
+ fd=OpenFileNew(nodesx->filename_tmp);
+
+ /* Modify the on-disk image */
+
+ while(!ReadFile(nodesx->fd,&nodex,sizeof(NodeX)))
+   {
+    if(segmentsx->firstnode[total]==NO_SEGMENT)
+      {
+       pruned++;
+
+       nodesx->pdata[total]=NO_NODE;
+      }
+    else
+      {
+       nodex.id=notpruned;
+       nodesx->pdata[total]=notpruned;
+
+       WriteFile(fd,&nodex,sizeof(NodeX));
+
+       notpruned++;
+      }
+
+    total++;
+
+    if(!(total%10000))
+       printf_middle("Deleting Pruned Nodes: Nodes=%"Pindex_t" Pruned=%"Pindex_t,total,pruned);
+   }
+
+ nodesx->number=notpruned;
+
+ /* Close the files */
+
+ nodesx->fd=CloseFile(nodesx->fd);
+ CloseFile(fd);
+
+ /* Print the final message */
+
+ printf_last("Deleted Pruned Nodes: Nodes=%"Pindex_t" Pruned=%"Pindex_t,total,pruned);
 }
 
 
@@ -271,6 +500,14 @@ static int deduplicate_and_index_by_id(NodeX *nodex,index_t index)
 void SortNodeListGeographically(NodesX *nodesx)
 {
  int fd;
+ ll_bin_t lat_min_bin,lat_max_bin,lon_min_bin,lon_max_bin;
+
+ /* While we are here we can work out the range of data */
+
+ lat_min=radians_to_latlong( 2);
+ lat_max=radians_to_latlong(-2);
+ lon_min=radians_to_latlong( 4);
+ lon_max=radians_to_latlong(-4);
 
  /* Print the start message */
 
@@ -280,30 +517,50 @@ void SortNodeListGeographically(NodesX *nodesx)
 
  nodesx->gdata=(index_t*)malloc(nodesx->number*sizeof(index_t));
 
- assert(nodesx->gdata); /* Check malloc() worked */
+ logassert(nodesx->gdata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
  /* Re-open the file read-only and a new file writeable */
 
- nodesx->fd=ReOpenFile(nodesx->filename);
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
 
- DeleteFile(nodesx->filename);
+ DeleteFile(nodesx->filename_tmp);
 
- fd=OpenFileNew(nodesx->filename);
+ fd=OpenFileNew(nodesx->filename_tmp);
 
- /* Sort geographically */
+ /* Sort nodes geographically and index them */
 
  sortnodesx=nodesx;
 
- filesort_fixed(nodesx->fd,fd,sizeof(NodeX),(int (*)(const void*,const void*))sort_by_lat_long,(int (*)(void*,index_t))index_by_lat_long);
+ filesort_fixed(nodesx->fd,fd,sizeof(NodeX),NULL,
+                                            (int (*)(const void*,const void*))sort_by_lat_long,
+                                            (int (*)(void*,index_t))index_by_lat_long);
 
  /* Close the files */
 
  nodesx->fd=CloseFile(nodesx->fd);
  CloseFile(fd);
 
+ /* Free the memory */
+
+ free(nodesx->super);
+ nodesx->super=NULL;
+
+ /* Work out the number of bins */
+
+ lat_min_bin=latlong_to_bin(lat_min);
+ lon_min_bin=latlong_to_bin(lon_min);
+ lat_max_bin=latlong_to_bin(lat_max);
+ lon_max_bin=latlong_to_bin(lon_max);
+
+ nodesx->latzero=lat_min_bin;
+ nodesx->lonzero=lon_min_bin;
+
+ nodesx->latbins=(lat_max_bin-lat_min_bin)+1;
+ nodesx->lonbins=(lon_max_bin-lon_min_bin)+1;
+
  /* Print the final message */
 
- printf_last("Sorted Nodes Geographically");
+ printf_last("Sorted Nodes Geographically: Nodes=%"Pindex_t,nodesx->number);
 }
 
 
@@ -351,7 +608,7 @@ static int sort_by_lat_long(NodeX *a,NodeX *b)
              return(1);
          }
 
-       return(0);
+       return(FILESORT_PRESERVE_ORDER(a,b));
       }
    }
 }
@@ -364,247 +621,40 @@ static int sort_by_lat_long(NodeX *a,NodeX *b)
 
   NodeX *nodex The extended node.
 
-  index_t index The index of this node in the total.
+  index_t index The number of sorted nodes that have already been written to the output file.
   ++++++++++++++++++++++++++++++++++++++*/
 
 static int index_by_lat_long(NodeX *nodex,index_t index)
 {
- /* Create the index from the previous sort to the current one */
-
  sortnodesx->gdata[nodex->id]=index;
 
+ if(IsBitSet(sortnodesx->super,nodex->id))
+    nodex->flags|=NODE_SUPER;
+
+ if(nodex->latitude<lat_min)
+    lat_min=nodex->latitude;
+ if(nodex->latitude>lat_max)
+    lat_max=nodex->latitude;
+ if(nodex->longitude<lon_min)
+    lon_min=nodex->longitude;
+ if(nodex->longitude>lon_max)
+    lon_max=nodex->longitude;
+
  return(1);
 }
 
 
-/*++++++++++++++++++++++++++++++++++++++
-  Find a particular node index.
-
-  index_t IndexNodeX Returns the index of the extended node with the specified id.
-
-  NodesX *nodesx The set of nodes to use.
-
-  node_t id The node id to look for.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-index_t IndexNodeX(NodesX *nodesx,node_t id)
-{
- index_t start=0;
- index_t end=nodesx->number-1;
- index_t mid;
-
- /* Binary search - search key exact match only is required.
-  *
-  *  # <- start  |  Check mid and move start or end if it doesn't match
-  *  #           |
-  *  #           |  Since an exact match is wanted we can set end=mid-1
-  *  # <- mid    |  or start=mid+1 because we know that mid doesn't match.
-  *  #           |
-  *  #           |  Eventually either end=start or end=start+1 and one of
-  *  # <- end    |  start or end is the wanted one.
-  */
-
- if(end<start)                        /* There are no nodes */
-    return(NO_NODE);
- else if(id<nodesx->idata[start])     /* Check key is not before start */
-    return(NO_NODE);
- else if(id>nodesx->idata[end])       /* Check key is not after end */
-    return(NO_NODE);
- else
-   {
-    do
-      {
-       mid=(start+end)/2;             /* Choose mid point */
-
-       if(nodesx->idata[mid]<id)      /* Mid point is too low */
-          start=mid+1;
-       else if(nodesx->idata[mid]>id) /* Mid point is too high */
-          end=mid-1;
-       else                           /* Mid point is correct */
-          return(mid);
-      }
-    while((end-start)>1);
-
-    if(nodesx->idata[start]==id)      /* Start is correct */
-       return(start);
-
-    if(nodesx->idata[end]==id)        /* End is correct */
-       return(end);
-   }
-
- return(NO_NODE);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Remove any nodes that are not part of a highway.
-
-  NodesX *nodesx The set of nodes to modify.
-
-  SegmentsX *segmentsx The set of segments to use.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-void RemoveNonHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx)
-{
- NodeX nodex;
- index_t total=0,highway=0,nothighway=0;
- ll_bin_t lat_min_bin,lat_max_bin,lon_min_bin,lon_max_bin;
- latlong_t lat_min,lat_max,lon_min,lon_max;
- int fd;
-
- /* Print the start message */
-
- printf_first("Checking Nodes: Nodes=0");
-
- /* While we are here we can work out the range of data */
-
- lat_min=radians_to_latlong( 2);
- lat_max=radians_to_latlong(-2);
- lon_min=radians_to_latlong( 4);
- lon_max=radians_to_latlong(-4);
-
- /* Re-open the file read-only and a new file writeable */
-
- nodesx->fd=ReOpenFile(nodesx->filename);
-
- DeleteFile(nodesx->filename);
-
- fd=OpenFileNew(nodesx->filename);
-
- /* Modify the on-disk image */
-
- while(!ReadFile(nodesx->fd,&nodex,sizeof(NodeX)))
-   {
-    if(!IsBitSet(segmentsx->usednode,total))
-       nothighway++;
-    else
-      {
-       nodex.id=highway;
-
-       WriteFile(fd,&nodex,sizeof(NodeX));
-
-       nodesx->idata[highway]=nodesx->idata[total];
-       highway++;
-
-       if(nodex.latitude<lat_min)
-          lat_min=nodex.latitude;
-       if(nodex.latitude>lat_max)
-          lat_max=nodex.latitude;
-       if(nodex.longitude<lon_min)
-          lon_min=nodex.longitude;
-       if(nodex.longitude>lon_max)
-          lon_max=nodex.longitude;
-      }
-
-    total++;
-
-    if(!(total%10000))
-       printf_middle("Checking Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
-   }
-
- nodesx->number=highway;
-
- /* Close the files */
-
- nodesx->fd=CloseFile(nodesx->fd);
- CloseFile(fd);
-
- /* Work out the number of bins */
-
- lat_min_bin=latlong_to_bin(lat_min);
- lon_min_bin=latlong_to_bin(lon_min);
- lat_max_bin=latlong_to_bin(lat_max);
- lon_max_bin=latlong_to_bin(lon_max);
-
- nodesx->latzero=lat_min_bin;
- nodesx->lonzero=lon_min_bin;
-
- nodesx->latbins=(lat_max_bin-lat_min_bin)+1;
- nodesx->lonbins=(lon_max_bin-lon_min_bin)+1;
-
- /* Free the now-unneeded index */
-
- free(segmentsx->usednode);
- segmentsx->usednode=NULL;
-
- /* Allocate and set the super-node markers */
-
- nodesx->super=(uint8_t*)malloc((1+nodesx->number/8)*sizeof(uint8_t));
-
- assert(nodesx->super); /* Check calloc() worked */
-
- memset(nodesx->super,~0,(1+nodesx->number/8));
-
- /* Print the final message */
-
- printf_last("Checked Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Insert the super-node flag and the first segment indexes after geographical sorting.
-
-  NodesX *nodesx The set of nodes to modify.
-
-  SegmentsX *segmentsx The set of segments to use.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-void UpdateNodes(NodesX *nodesx,SegmentsX *segmentsx)
-{
- index_t i;
- int fd;
-
- /* Print the start message */
-
- printf_first("Updating Super Nodes: Nodes=0");
-
- /* Re-open the file read-only and a new file writeable */
-
- nodesx->fd=ReOpenFile(nodesx->filename);
-
- DeleteFile(nodesx->filename);
-
- fd=OpenFileNew(nodesx->filename);
-
- /* Modify the on-disk image */
-
- for(i=0;i<nodesx->number;i++)
-   {
-    NodeX nodex;
-
-    ReadFile(nodesx->fd,&nodex,sizeof(NodeX));
-
-    if(IsBitSet(nodesx->super,nodex.id))
-       nodex.flags|=NODE_SUPER;
-
-    nodex.id=segmentsx->firstnode[nodesx->gdata[nodex.id]];
-
-    WriteFile(fd,&nodex,sizeof(NodeX));
-
-    if(!((i+1)%10000))
-       printf_middle("Updating Super Nodes: Nodes=%"Pindex_t,i+1);
-   }
-
- /* Close the files */
-
- nodesx->fd=CloseFile(nodesx->fd);
- CloseFile(fd);
-
- /* Print the final message */
-
- printf_last("Updated Super Nodes: Nodes=%"Pindex_t,nodesx->number);
-}
-
-
 /*++++++++++++++++++++++++++++++++++++++
   Save the final node list database to a file.
 
   NodesX *nodesx The set of nodes to save.
 
   const char *filename The name of the file to save.
+
+  SegmentsX *segmentsx The set of segments to use.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void SaveNodeList(NodesX *nodesx,const char *filename)
+void SaveNodeList(NodesX *nodesx,const char *filename,SegmentsX *segmentsx)
 {
  index_t i;
  int fd;
@@ -621,13 +671,13 @@ void SaveNodeList(NodesX *nodesx,const char *filename)
 
  offsets=(index_t*)malloc((nodesx->latbins*nodesx->lonbins+1)*sizeof(index_t));
 
- assert(offsets); /* Check malloc() worked */
+ logassert(offsets,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
  latlonbin=0;
 
  /* Re-open the file */
 
- nodesx->fd=ReOpenFile(nodesx->filename);
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
 
  /* Write out the nodes data */
 
@@ -638,7 +688,7 @@ void SaveNodeList(NodesX *nodesx,const char *filename)
  for(i=0;i<nodesx->number;i++)
    {
     NodeX nodex;
-    Node node;
+    Node node={0};
     ll_bin_t latbin,lonbin;
     ll_bin2_t llbin;
 
@@ -648,7 +698,7 @@ void SaveNodeList(NodesX *nodesx,const char *filename)
 
     node.latoffset=latlong_to_off(nodex.latitude);
     node.lonoffset=latlong_to_off(nodex.longitude);
-    node.firstseg=nodex.id;
+    node.firstseg=segmentsx->firstnode[nodesx->gdata[nodex.id]];
     node.allow=nodex.allow;
     node.flags=nodex.flags;
 
