@@ -20,7 +20,6 @@
  ***************************************/
 
 
-#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -28,6 +27,7 @@
 #include "ways.h"
 
 #include "typesx.h"
+#include "segmentsx.h"
 #include "waysx.h"
 
 #include "files.h"
@@ -40,20 +40,23 @@
 /*+ The command line '--tmpdir' option or its default value. +*/
 extern char *option_tmpdirname;
 
-
 /* Local variables */
 
-/*+ A temporary file-local variable for use by the sort functions. +*/
+/*+ Temporary file-local variables for use by the sort functions. +*/
 static WaysX *sortwaysx;
+static SegmentsX *sortsegmentsx;
 
-
-/* Functions */
+/* Local functions */
 
 static int sort_by_id(WayX *a,WayX *b);
-static int sort_by_name_and_id(WayX *a,WayX *b);
-static int sort_by_name_and_prop_and_id(WayX *a,WayX *b);
+static int deduplicate_by_id(WayX *wayx,index_t index);
 
-static int deduplicate_and_index_by_id(WayX *wayx,index_t index);
+static int sort_by_name(WayX *a,WayX *b);
+static int index_by_id(WayX *wayx,index_t index);
+
+static int delete_unused(WayX *wayx,index_t index);
+static int sort_by_name_and_prop_and_id(WayX *a,WayX *b);
+static int deduplicate_and_index_by_compact_id(WayX *wayx,index_t index);
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -61,49 +64,61 @@ static int deduplicate_and_index_by_id(WayX *wayx,index_t index);
 
   WaysX *NewWayList Returns the way list.
 
-  int append Set to 1 if the file is to be opened for appending (now or later).
+  int append Set to 1 if the file is to be opened for appending.
+
+  int readonly Set to 1 if the file is to be opened for reading.
   ++++++++++++++++++++++++++++++++++++++*/
 
-WaysX *NewWayList(int append)
+WaysX *NewWayList(int append,int readonly)
 {
  WaysX *waysx;
 
  waysx=(WaysX*)calloc(1,sizeof(WaysX));
 
- assert(waysx); /* Check calloc() worked */
+ logassert(waysx,"Failed to allocate memory (try using slim mode?)"); /* Check calloc() worked */
 
- waysx->filename=(char*)malloc(strlen(option_tmpdirname)+32);
+ waysx->filename    =(char*)malloc(strlen(option_tmpdirname)+32);
+ waysx->filename_tmp=(char*)malloc(strlen(option_tmpdirname)+32);
 
- if(append)
-    sprintf(waysx->filename,"%s/waysx.input.tmp",option_tmpdirname);
- else
-    sprintf(waysx->filename,"%s/waysx.%p.tmp",option_tmpdirname,(void*)waysx);
+ sprintf(waysx->filename    ,"%s/waysx.parsed.mem",option_tmpdirname);
+ sprintf(waysx->filename_tmp,"%s/waysx.%p.tmp"    ,option_tmpdirname,(void*)waysx);
 
- if(append)
-   {
-    off_t size,position=0;
-
-    waysx->fd=OpenFileAppend(waysx->filename);
-
-    size=SizeFile(waysx->filename);
-
-    while(position<size)
+ if(append || readonly)
+    if(ExistsFile(waysx->filename))
       {
-       FILESORT_VARINT waysize;
+       off_t size,position=0;
+       int fd;
 
-       SeekReadFile(waysx->fd,&waysize,FILESORT_VARSIZE,position);
+       size=SizeFile(waysx->filename);
 
-       waysx->number++;
-       position+=waysize+FILESORT_VARSIZE;
+       fd=ReOpenFile(waysx->filename);
+
+       while(position<size)
+         {
+          FILESORT_VARINT waysize;
+
+          SeekReadFile(fd,&waysize,FILESORT_VARSIZE,position);
+
+          waysx->number++;
+          position+=waysize+FILESORT_VARSIZE;
+         }
+
+       CloseFile(fd);
+
+       RenameFile(waysx->filename,waysx->filename_tmp);
       }
 
-    SeekFile(waysx->fd,size);
-   }
+ if(append)
+    waysx->fd=OpenFileAppend(waysx->filename_tmp);
+ else if(!readonly)
+    waysx->fd=OpenFileNew(waysx->filename_tmp);
  else
-    waysx->fd=OpenFileNew(waysx->filename);
+    waysx->fd=-1;
 
- waysx->nfilename=(char*)malloc(strlen(option_tmpdirname)+32);
- sprintf(waysx->nfilename,"%s/waynames.%p.tmp",option_tmpdirname,(void*)waysx);
+
+ waysx->nfilename_tmp=(char*)malloc(strlen(option_tmpdirname)+32);
+
+ sprintf(waysx->nfilename_tmp,"%s/waynames.%p.tmp",option_tmpdirname,(void*)waysx);
 
  return(waysx);
 }
@@ -114,22 +129,28 @@ WaysX *NewWayList(int append)
 
   WaysX *waysx The set of ways to be freed.
 
-  int keep Set to 1 if the file is to be kept (for appending later).
+  int keep If set then the results file is to be kept.
   ++++++++++++++++++++++++++++++++++++++*/
 
 void FreeWayList(WaysX *waysx,int keep)
 {
- if(!keep)
-    DeleteFile(waysx->filename);
+ if(keep)
+    RenameFile(waysx->filename_tmp,waysx->filename);
+ else
+    DeleteFile(waysx->filename_tmp);
 
  free(waysx->filename);
+ free(waysx->filename_tmp);
 
  if(waysx->idata)
     free(waysx->idata);
 
- DeleteFile(waysx->nfilename);
+ if(waysx->cdata)
+    free(waysx->cdata);
 
- free(waysx->nfilename);
+ DeleteFile(waysx->nfilename_tmp);
+
+ free(waysx->nfilename_tmp);
 
  free(waysx);
 }
@@ -147,13 +168,12 @@ void FreeWayList(WaysX *waysx,int keep)
   const char *name The name or reference of the way.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void AppendWay(WaysX *waysx,way_t id,Way *way,const char *name)
+void AppendWayList(WaysX *waysx,way_t id,Way *way,const char *name)
 {
  WayX wayx;
  FILESORT_VARINT size;
 
  wayx.id=id;
- wayx.prop=0;
  wayx.way=*way;
 
  size=sizeof(WayX)+strlen(name)+1;
@@ -164,374 +184,20 @@ void AppendWay(WaysX *waysx,way_t id,Way *way,const char *name)
 
  waysx->number++;
 
- assert(waysx->number!=0); /* Zero marks the high-water mark for ways. */
+ logassert(waysx->number!=0,"Too many ways (change index_t to 64-bits?)"); /* Zero marks the high-water mark for ways. */
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Sort the list of ways.
+  Finish appending ways and change the filename over.
 
-  WaysX *waysx The set of ways to process.
+  WaysX *waysx The ways that have been appended.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void SortWayList(WaysX *waysx)
+void FinishWayList(WaysX *waysx)
 {
- index_t i,xnumber;
- int fd;
- char *names[2]={NULL,NULL};
- int namelen[2]={0,0};
- int nnames=0;
- uint32_t lastlength=0;
-
- /* Print the start message */
-
- printf_first("Sorting Ways by Name");
-
- /* Close the file (finished appending) */
-
- waysx->fd=CloseFile(waysx->fd);
-
- /* Re-open the file read-only and a new file writeable */
-
- waysx->fd=ReOpenFile(waysx->filename);
-
- DeleteFile(waysx->filename);
-
- fd=OpenFileNew(waysx->filename);
-
- /* Sort the ways to allow separating the names */
-
- filesort_vary(waysx->fd,fd,(int (*)(const void*,const void*))sort_by_name_and_id,NULL);
-
- /* Close the files */
-
- waysx->fd=CloseFile(waysx->fd);
- CloseFile(fd);
-
- /* Print the final message */
-
- printf_last("Sorted Ways by Name: Ways=%"Pindex_t,waysx->number);
-
-
- /* Print the start message */
-
- printf_first("Separating Way Names: Ways=0 Names=0");
-
- /* Re-open the file read-only and new files writeable */
-
- waysx->fd=ReOpenFile(waysx->filename);
-
- DeleteFile(waysx->filename);
-
- fd=OpenFileNew(waysx->filename);
-
- waysx->nfd=OpenFileNew(waysx->nfilename);
-
- /* Copy from the single file into two files */
-
- for(i=0;i<waysx->number;i++)
-   {
-    WayX wayx;
-    FILESORT_VARINT size;
-
-    ReadFile(waysx->fd,&size,FILESORT_VARSIZE);
-
-    if(namelen[nnames%2]<size)
-       names[nnames%2]=(char*)realloc((void*)names[nnames%2],namelen[nnames%2]=size);
-
-    ReadFile(waysx->fd,&wayx,sizeof(WayX));
-    ReadFile(waysx->fd,names[nnames%2],size-sizeof(WayX));
-
-    if(nnames==0 || strcmp(names[0],names[1]))
-      {
-       WriteFile(waysx->nfd,names[nnames%2],size-sizeof(WayX));
-
-       lastlength=waysx->nlength;
-       waysx->nlength+=size-sizeof(WayX);
-
-       nnames++;
-      }
-
-    wayx.way.name=lastlength;
-
-    WriteFile(fd,&wayx,sizeof(WayX));
-
-    if(!((i+1)%1000))
-       printf_middle("Separating Way Names: Ways=%"Pindex_t" Names=%"Pindex_t,i+1,nnames);
-   }
-
- if(names[0]) free(names[0]);
- if(names[1]) free(names[1]);
-
- /* Close the files */
-
- waysx->fd=CloseFile(waysx->fd);
- CloseFile(fd);
-
- waysx->nfd=CloseFile(waysx->nfd);
-
- /* Print the final message */
-
- printf_last("Separated Way Names: Ways=%"Pindex_t" Names=%"Pindex_t" ",waysx->number,nnames);
-
-
- /* Print the start message */
-
- printf_first("Sorting Ways");
-
- /* Re-open the file read-only and a new file writeable */
-
- waysx->fd=ReOpenFile(waysx->filename);
-
- DeleteFile(waysx->filename);
-
- fd=OpenFileNew(waysx->filename);
-
- /* Allocate the array of indexes */
-
- waysx->idata=(way_t*)malloc(waysx->number*sizeof(way_t));
-
- assert(waysx->idata); /* Check malloc() worked */
-
- /* Sort the ways by index and index them */
-
- xnumber=waysx->number;
- waysx->number=0;
-
- sortwaysx=waysx;
-
- waysx->number=filesort_fixed(waysx->fd,fd,sizeof(WayX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))deduplicate_and_index_by_id);
-
- /* Close the files */
-
- waysx->fd=CloseFile(waysx->fd);
- CloseFile(fd);
-
- /* Print the final message */
-
- printf_last("Sorted Ways: Ways=%"Pindex_t" Duplicates=%"Pindex_t,xnumber,xnumber-waysx->number);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Compact the list of ways.
-
-  WaysX *waysx The set of ways to process.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-void CompactWayList(WaysX *waysx)
-{
- index_t i;
- int fd;
- Way lastway;
-
- /* Print the start message */
-
- printf_first("Sorting Ways by Properties");
-
- /* Re-open the file read-only and a new file writeable */
-
- waysx->fd=ReOpenFile(waysx->filename);
-
- DeleteFile(waysx->filename);
-
- fd=OpenFileNew(waysx->filename);
-
- /* Sort the ways to allow compacting according to he properties */
-
- filesort_fixed(waysx->fd,fd,sizeof(WayX),(int (*)(const void*,const void*))sort_by_name_and_prop_and_id,NULL);
-
- /* Close the files */
-
- waysx->fd=CloseFile(waysx->fd);
- CloseFile(fd);
-
- /* Print the final message */
-
- printf_last("Sorted Ways by Properties: Ways=%"Pindex_t,waysx->number);
-
-
- /* Print the start message */
-
- printf_first("Compacting Ways: Ways=0 Properties=0");
-
- /* Re-open the file read-only and a new file writeable */
-
- waysx->fd=ReOpenFile(waysx->filename);
-
- DeleteFile(waysx->filename);
-
- fd=OpenFileNew(waysx->filename);
-
- /* Update the way as we go using the sorted index */
-
- waysx->cnumber=0;
-
- for(i=0;i<waysx->number;i++)
-   {
-    WayX wayx;
-
-    ReadFile(waysx->fd,&wayx,sizeof(WayX));
-
-    if(waysx->cnumber==0 || wayx.way.name!=lastway.name || WaysCompare(&lastway,&wayx.way))
-      {
-       lastway=wayx.way;
-
-       waysx->cnumber++;
-      }
-
-    wayx.prop=waysx->cnumber-1;
-
-    WriteFile(fd,&wayx,sizeof(WayX));
-
-    if(!((i+1)%1000))
-       printf_middle("Compacting Ways: Ways=%"Pindex_t" Properties=%"Pindex_t,i+1,waysx->cnumber);
-   }
-
- /* Close the files */
-
- waysx->fd=CloseFile(waysx->fd);
- CloseFile(fd);
-
- /* Print the final message */
-
- printf_last("Compacted Ways: Ways=%"Pindex_t" Properties=%"Pindex_t" ",waysx->number,waysx->cnumber);
-
-
- /* Print the start message */
-
- printf_first("Sorting Ways");
-
- /* Re-open the file read-only and a new file writeable */
-
- waysx->fd=ReOpenFile(waysx->filename);
-
- DeleteFile(waysx->filename);
-
- fd=OpenFileNew(waysx->filename);
-
- /* Sort the ways by index */
-
- filesort_fixed(waysx->fd,fd,sizeof(WayX),(int (*)(const void*,const void*))sort_by_id,NULL);
-
- /* Close the files */
-
- waysx->fd=CloseFile(waysx->fd);
- CloseFile(fd);
-
- /* Print the final message */
-
- printf_last("Sorted Ways: Ways=%"Pindex_t,waysx->number);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Sort the ways into id order.
-
-  int sort_by_id Returns the comparison of the id fields.
-
-  WayX *a The first extended way.
-
-  WayX *b The second extended way.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int sort_by_id(WayX *a,WayX *b)
-{
- way_t a_id=a->id;
- way_t b_id=b->id;
-
- if(a_id<b_id)
-    return(-1);
- else if(a_id>b_id)
-    return(1);
- else
-    return(0);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Sort the ways into name order and then id order.
-
-  int sort_by_name_and_id Returns the comparison of the name and id fields.
-
-  WayX *a The first extended Way.
-
-  WayX *b The second extended Way.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int sort_by_name_and_id(WayX *a,WayX *b)
-{
- int compare;
- char *a_name=(char*)a+sizeof(WayX);
- char *b_name=(char*)b+sizeof(WayX);
-
- compare=strcmp(a_name,b_name);
-
- if(compare)
-    return(compare);
-
- return(sort_by_id(a,b));
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Sort the ways into name, properties and id order.
-
-  int sort_by_name_and_prop_and_id Returns the comparison of the name, properties and id fields.
-
-  WayX *a The first extended Way.
-
-  WayX *b The second extended Way.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int sort_by_name_and_prop_and_id(WayX *a,WayX *b)
-{
- int compare;
- index_t a_name=a->way.name;
- index_t b_name=b->way.name;
-
- if(a_name<b_name)
-    return(-1);
- else if(a_name>b_name)
-    return(1);
-
- compare=WaysCompare(&a->way,&b->way);
-
- if(compare)
-    return(compare);
-
- return(sort_by_id(a,b));
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Create the index of identifiers and discard duplicate ways.
-
-  int deduplicate_and_index_by_id Return 1 if the value is to be kept, otherwise 0.
-
-  WayX *wayx The extended way.
-
-  index_t index The index of this way in the total.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int deduplicate_and_index_by_id(WayX *wayx,index_t index)
-{
- static way_t previd;
-
- if(index==0 || wayx->id!=previd)
-   {
-    previd=wayx->id;
-
-    sortwaysx->idata[index]=wayx->id;
-
-    return(1);
-   }
- else
-   {
-    logerror("Way %"Pway_t" is duplicated.\n",wayx->id);
-
-    return(0);
-   }
+ if(waysx->fd!=-1)
+    waysx->fd=CloseFile(waysx->fd);
 }
 
 
@@ -594,6 +260,435 @@ index_t IndexWayX(WaysX *waysx,way_t id)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Sort the list of ways.
+
+  WaysX *waysx The set of ways to process.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void SortWayList(WaysX *waysx)
+{
+ index_t xnumber;
+ int fd;
+
+ /* Print the start message */
+
+ printf_first("Sorting Ways");
+
+ /* Re-open the file read-only and a new file writeable */
+
+ waysx->fd=ReOpenFile(waysx->filename_tmp);
+
+ DeleteFile(waysx->filename_tmp);
+
+ fd=OpenFileNew(waysx->filename_tmp);
+
+ /* Sort the ways by ID and index them */
+
+ xnumber=waysx->number;
+
+ waysx->number=filesort_vary(waysx->fd,fd,NULL,
+                                          (int (*)(const void*,const void*))sort_by_id,
+                                          (int (*)(void*,index_t))deduplicate_by_id);
+
+ /* Close the files */
+
+ waysx->fd=CloseFile(waysx->fd);
+ CloseFile(fd);
+
+ /* Print the final message */
+
+ printf_last("Sorted Ways: Ways=%"Pindex_t" Duplicates=%"Pindex_t,xnumber,xnumber-waysx->number);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Sort the ways into id order.
+
+  int sort_by_id Returns the comparison of the id fields.
+
+  WayX *a The first extended way.
+
+  WayX *b The second extended way.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int sort_by_id(WayX *a,WayX *b)
+{
+ way_t a_id=a->id;
+ way_t b_id=b->id;
+
+ if(a_id<b_id)
+    return(-1);
+ else if(a_id>b_id)
+    return(1);
+ else
+    return(-FILESORT_PRESERVE_ORDER(a,b)); /* latest version first */
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Discard duplicate ways.
+
+  int deduplicate_by_id Return 1 if the value is to be kept, otherwise 0.
+
+  WayX *wayx The extended way.
+
+  index_t index The number of sorted ways that have already been written to the output file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int deduplicate_by_id(WayX *wayx,index_t index)
+{
+ static way_t previd=NO_WAY_ID;
+
+ if(wayx->id!=previd)
+   {
+    previd=wayx->id;
+
+    if(wayx->way.type==WAY_DELETED)
+       return(0);
+    else
+       return(1);
+   }
+ else
+    return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Extract the way names from the ways and reference the list of names from the ways.
+
+  WaysX *waysx The set of ways to process.
+
+  int keep If set to 1 then keep the old data file otherwise delete it.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void ExtractWayNames(WaysX *waysx,int keep)
+{
+ index_t i;
+ int fd;
+ char *names[2]={NULL,NULL};
+ int namelen[2]={0,0};
+ int nnames=0;
+ uint32_t lastlength=0;
+
+ /* Print the start message */
+
+ printf_first("Sorting Ways by Name");
+
+ /* Re-open the file read-only and a new file writeable */
+
+ waysx->fd=ReOpenFile(waysx->filename_tmp);
+
+ if(keep)
+    RenameFile(waysx->filename_tmp,waysx->filename);
+ else
+    DeleteFile(waysx->filename_tmp);
+
+ fd=OpenFileNew(waysx->filename_tmp);
+
+ /* Sort the ways to allow separating the names */
+
+ filesort_vary(waysx->fd,fd,NULL,
+                            (int (*)(const void*,const void*))sort_by_name,
+                            NULL);
+
+ /* Close the files */
+
+ waysx->fd=CloseFile(waysx->fd);
+ CloseFile(fd);
+
+ /* Print the final message */
+
+ printf_last("Sorted Ways by Name: Ways=%"Pindex_t,waysx->number);
+
+
+ /* Print the start message */
+
+ printf_first("Separating Way Names: Ways=0 Names=0");
+
+ /* Re-open the file read-only and new files writeable */
+
+ waysx->fd=ReOpenFile(waysx->filename_tmp);
+
+ DeleteFile(waysx->filename_tmp);
+
+ fd=OpenFileNew(waysx->filename_tmp);
+
+ waysx->nfd=OpenFileNew(waysx->nfilename_tmp);
+
+ /* Copy from the single file into two files */
+
+ for(i=0;i<waysx->number;i++)
+   {
+    WayX wayx;
+    FILESORT_VARINT size;
+
+    ReadFile(waysx->fd,&size,FILESORT_VARSIZE);
+
+    if(namelen[nnames%2]<size)
+       names[nnames%2]=(char*)realloc((void*)names[nnames%2],namelen[nnames%2]=size);
+
+    ReadFile(waysx->fd,&wayx,sizeof(WayX));
+    ReadFile(waysx->fd,names[nnames%2],size-sizeof(WayX));
+
+    if(nnames==0 || strcmp(names[0],names[1]))
+      {
+       WriteFile(waysx->nfd,names[nnames%2],size-sizeof(WayX));
+
+       lastlength=waysx->nlength;
+       waysx->nlength+=size-sizeof(WayX);
+
+       nnames++;
+      }
+
+    wayx.way.name=lastlength;
+
+    WriteFile(fd,&wayx,sizeof(WayX));
+
+    if(!((i+1)%1000))
+       printf_middle("Separating Way Names: Ways=%"Pindex_t" Names=%"Pindex_t,i+1,nnames);
+   }
+
+ if(names[0]) free(names[0]);
+ if(names[1]) free(names[1]);
+
+ /* Close the files */
+
+ waysx->fd=CloseFile(waysx->fd);
+ CloseFile(fd);
+
+ waysx->nfd=CloseFile(waysx->nfd);
+
+ /* Print the final message */
+
+ printf_last("Separated Way Names: Ways=%"Pindex_t" Names=%"Pindex_t,waysx->number,nnames);
+
+
+ /* Print the start message */
+
+ printf_first("Sorting Ways");
+
+ /* Re-open the file read-only and a new file writeable */
+
+ waysx->fd=ReOpenFile(waysx->filename_tmp);
+
+ DeleteFile(waysx->filename_tmp);
+
+ fd=OpenFileNew(waysx->filename_tmp);
+
+ /* Allocate the array of indexes */
+
+ waysx->idata=(way_t*)malloc(waysx->number*sizeof(way_t));
+
+ logassert(waysx->idata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
+
+ /* Sort the ways by ID */
+
+ sortwaysx=waysx;
+
+ filesort_fixed(waysx->fd,fd,sizeof(WayX),NULL,
+                                          (int (*)(const void*,const void*))sort_by_id,
+                                          (int (*)(void*,index_t))index_by_id);
+
+ /* Close the files */
+
+ waysx->fd=CloseFile(waysx->fd);
+ CloseFile(fd);
+
+ /* Print the final message */
+
+ printf_last("Sorted Ways: Ways=%"Pindex_t,waysx->number);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Sort the ways into name order and then id order.
+
+  int sort_by_name Returns the comparison of the name fields.
+
+  WayX *a The first extended Way.
+
+  WayX *b The second extended Way.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int sort_by_name(WayX *a,WayX *b)
+{
+ int compare;
+ char *a_name=(char*)a+sizeof(WayX);
+ char *b_name=(char*)b+sizeof(WayX);
+
+ compare=strcmp(a_name,b_name);
+
+ if(compare)
+    return(compare);
+ else
+    return(FILESORT_PRESERVE_ORDER(a,b));
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Create the index of identifiers.
+
+  int index_by_id Return 1 if the value is to be kept, otherwise 0.
+
+  WayX *wayx The extended way.
+
+  index_t index The number of sorted ways that have already been written to the output file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int index_by_id(WayX *wayx,index_t index)
+{
+ sortwaysx->idata[index]=wayx->id;
+
+ return(1);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Compact the way list, removing duplicated ways and unused ways.
+
+  WaysX *waysx The set of ways to process.
+
+  SegmentsX *segmentsx The set of segments to check.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void CompactWayList(WaysX *waysx,SegmentsX *segmentsx)
+{
+ int fd;
+ index_t cnumber;
+
+ /* Print the start message */
+
+ printf_first("Sorting Ways and Compacting");
+
+ /* Allocate the array of indexes */
+
+ waysx->cdata=(index_t*)malloc(waysx->number*sizeof(index_t));
+
+ logassert(waysx->cdata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
+
+ /* Re-open the file read-only and a new file writeable */
+
+ waysx->fd=ReOpenFile(waysx->filename_tmp);
+
+ DeleteFile(waysx->filename_tmp);
+
+ fd=OpenFileNew(waysx->filename_tmp);
+
+ /* Sort the ways to allow compacting according to the properties */
+
+ sortwaysx=waysx;
+ sortsegmentsx=segmentsx;
+
+ cnumber=filesort_fixed(waysx->fd,fd,sizeof(WayX),(int (*)(void*,index_t))delete_unused,
+                                                  (int (*)(const void*,const void*))sort_by_name_and_prop_and_id,
+                                                  (int (*)(void*,index_t))deduplicate_and_index_by_compact_id);
+
+ /* Close the files */
+
+ waysx->fd=CloseFile(waysx->fd);
+ CloseFile(fd);
+
+ /* Print the final message */
+
+ printf_last("Sorted and Compacted Ways: Ways=%"Pindex_t" Unique=%"Pindex_t,waysx->number,cnumber);
+ waysx->number=cnumber;
+
+ free(segmentsx->usedway);
+ segmentsx->usedway=NULL;
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Delete the ways that are no longer being used.
+
+  int delete_unused Return 1 if the value is to be kept, otherwise 0.
+
+  WayX *wayx The extended way.
+
+  index_t index The number of unsorted ways that have been read from the input file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int delete_unused(WayX *wayx,index_t index)
+{
+ if(sortsegmentsx && !IsBitSet(sortsegmentsx->usedway,index))
+   {
+    sortwaysx->cdata[index]=NO_WAY;
+
+    return(0);
+   }
+ else
+   {
+    wayx->id=index;
+
+    return(1);
+   }
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Sort the ways into name, properties and id order.
+
+  int sort_by_name_and_prop_and_id Returns the comparison of the name, properties and id fields.
+
+  WayX *a The first extended Way.
+
+  WayX *b The second extended Way.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int sort_by_name_and_prop_and_id(WayX *a,WayX *b)
+{
+ int compare;
+ index_t a_name=a->way.name;
+ index_t b_name=b->way.name;
+
+ if(a_name<b_name)
+    return(-1);
+ else if(a_name>b_name)
+    return(1);
+
+ compare=WaysCompare(&a->way,&b->way);
+
+ if(compare)
+    return(compare);
+
+ return(sort_by_id(a,b));
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Create the index of compacted Way identifiers and ignore Ways with duplicated properties.
+
+  int deduplicate_and_index_by_compact_id Return 1 if the value is to be kept, otherwise 0.
+
+  WayX *wayx The extended way.
+
+  index_t index The number of sorted ways that have already been written to the output file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int deduplicate_and_index_by_compact_id(WayX *wayx,index_t index)
+{
+ static Way lastway;
+
+ if(index==0 || wayx->way.name!=lastway.name || WaysCompare(&lastway,&wayx->way))
+   {
+    lastway=wayx->way;
+
+    sortwaysx->cdata[wayx->id]=index;
+
+    wayx->id=index;
+
+    return(1);
+   }
+ else
+   {
+    sortwaysx->cdata[wayx->id]=index-1;
+
+    return(0);
+   }
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Save the way list to a file.
 
@@ -607,6 +702,7 @@ void SaveWayList(WaysX *waysx,const char *filename)
  index_t i;
  int fd;
  int position=0;
+ WayX wayx;
  WaysFile waysfile={0};
  highways_t   highways=0;
  transports_t allow=0;
@@ -616,13 +712,10 @@ void SaveWayList(WaysX *waysx,const char *filename)
 
  printf_first("Writing Ways: Ways=0");
 
- /* Map into memory /  open the file */
+ /* Re-open the files */
 
-#if !SLIM
- waysx->data=MapFile(waysx->filename);
-#else
- waysx->fd=ReOpenFile(waysx->filename);
-#endif
+ waysx->fd=ReOpenFile(waysx->filename_tmp);
+ waysx->nfd=ReOpenFile(waysx->nfilename_tmp);
 
  /* Write out the ways data */
 
@@ -632,31 +725,21 @@ void SaveWayList(WaysX *waysx,const char *filename)
 
  for(i=0;i<waysx->number;i++)
    {
-    WayX *wayx=LookupWayX(waysx,i,1);
+    ReadFile(waysx->fd,&wayx,sizeof(WayX));
 
-    highways|=HIGHWAYS(wayx->way.type);
-    allow   |=wayx->way.allow;
-    props   |=wayx->way.props;
+    highways|=HIGHWAYS(wayx.way.type);
+    allow   |=wayx.way.allow;
+    props   |=wayx.way.props;
 
-    SeekWriteFile(fd,&wayx->way,sizeof(Way),sizeof(WaysFile)+(off_t)wayx->prop*sizeof(Way));
+    WriteFile(fd,&wayx.way,sizeof(Way));
 
     if(!((i+1)%1000))
        printf_middle("Writing Ways: Ways=%"Pindex_t,i+1);
    }
 
- /* Unmap from memory / close the file */
-
-#if !SLIM
- waysx->data=UnmapFile(waysx->filename);
-#else
- waysx->fd=CloseFile(waysx->fd);
-#endif
-
  /* Write out the ways names */
 
- SeekFile(fd,sizeof(WaysFile)+(off_t)waysx->cnumber*sizeof(Way));
-
- waysx->nfd=ReOpenFile(waysx->nfilename);
+ SeekFile(fd,sizeof(WaysFile)+(off_t)waysx->number*sizeof(Way));
 
  while(position<waysx->nlength)
    {
@@ -667,19 +750,20 @@ void SaveWayList(WaysX *waysx,const char *filename)
        len=waysx->nlength-position;
 
     ReadFile(waysx->nfd,temp,len);
+
     WriteFile(fd,temp,len);
 
     position+=len;
    }
 
- /* Close the file */
+ /* Close the files */
 
+ waysx->fd=CloseFile(waysx->fd);
  waysx->nfd=CloseFile(waysx->nfd);
 
  /* Write out the header structure */
 
- waysfile.number =waysx->cnumber;
- waysfile.onumber=waysx->number;
+ waysfile.number =waysx->number;
 
  waysfile.highways=highways;
  waysfile.allow   =allow;
